package geniusweb.issuevalue;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Contains a (possibly partial) bid. Basically a map. immutable, thread safe.
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE)
public class Bid {
	// not immutable and explicit HashMap for better serializability, must be
	// kept immutable.
	private final HashMap<String, Value> issuevalues;

	/**
	 * 
	 * @param issuevalues a map of issue, Value pairs. The String is the issue
	 *                    name.
	 */
	@JsonCreator
	public Bid(@JsonProperty("issuevalues") Map<String, Value> issuevalues) {
		if (issuevalues == null) {
			throw new NullPointerException("issuevalues=null");
		}
		for (String issue : issuevalues.keySet()) {
			if (issuevalues.get(issue) == null) {
				throw new IllegalArgumentException(
						"value of issue " + issue + " can not be null");
			}
		}
		this.issuevalues = new HashMap<String, Value>(issuevalues);
	}

	/**
	 * @param issue
	 * @return the value for the given issue, or null if there is no value for
	 *         the given issue.
	 */
	public Value getValue(String issue) {
		return issuevalues.get(issue);
	}

	/**
	 * 
	 * @param issue
	 * @return true iff the bid contains a value for the given issue.
	 */
	public boolean containsIssue(String issue) {
		return issuevalues.containsKey(issue);
	}

	public Set<String> getIssues() {
		return Collections.unmodifiableSet(issuevalues.keySet());
	}

	@Override
	public String toString() {
		return "Bid" + issuevalues;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((issuevalues == null) ? 0 : issuevalues.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Bid other = (Bid) obj;
		if (issuevalues == null) {
			if (other.issuevalues != null)
				return false;
		} else if (!issuevalues.equals(other.issuevalues))
			return false;
		return true;
	}

}